<!---
The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 
Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.
 
The Original Code is comprised of the ADF directory
 
The Initial Developer of the Original Code is
PaperThin, Inc.  Copyright (c) 2009-2018.
All Rights Reserved.
 
By downloading, modifying, distributing, using and/or accessing any files
in this directory, you agree to the terms and conditions of the applicable
end user license agreement.
--->

<!---
/* *************************************************************** */
Author: 	
	PaperThin Inc.
Name:
	field_settings_props.cfm
Summary:
	Renders input controls to capture field setting values.
Version:
	1.0.0
History:
	2017-11-29 - Created
	2017-12-08 - DRM - Added units support for padding
	2017-12-12 - JTP - Added units title text to selection list in render file
--->
<cfsetting enablecfoutputonly="Yes" showdebugoutput="No">

<!--- // if this module loads resources, do it here.. --->
<!---<cfscript>
    // No resources to load
</cfscript>--->

<!--- ... then exit if all we're doing is detecting required resources --->
<cfif Request.RenderState.RenderMode EQ "getResources">
  <cfexit>
</cfif>

<cfscript>
	prefix = attributes.prefix;
	currentValues = attributes.currentValues;
	typeid = attributes.typeid;
	formname = attributes.formname;	

	// writedump( var="#currentValues#", label="currentValues", expand="no" );
	
	// Variable for the version of the field - Display in Props UI.
	fieldVersion = "1.0.3";
	
	// initialize some of the attributes variables
	//Setup the default values
	gProps = 'show,tag,align,size,fontsettings,color,bgcolor,bordercolor,style,effect,padding';

	defaultValues = StructNew();
	
	for( i=1; i lte ListLen(gProps); i=i+1 )
	{
		prop = ListGetAt(gProps, i);
		defaultValues['#prop#Label'] = "";
		defaultValues['#prop#Value'] = "";
		defaultValues['#prop#Default'] = "";
		defaultValues['display#prop#'] = 0;
	}	
	defaultValues.showValue = "Yes,No";
	defaultValues.bgcolorLabel = "Background Color";
	defaultValues.borderColorLabel = "Border Color";
	defaultValues.fontSettingsLabel = "Font Settings";

   //This will override the default values with the current values.
   currentValueArray = StructKeyArray(currentValues);
   for(i=1; i<=ArrayLen(currentValueArray); i++)
	{
		if(StructKeyExists(defaultValues,currentValueArray[i]))
			defaultValues[currentValueArray[i]] = currentValues[currentValueArray[i]];
	}
	
	// Build paramFields list from defautValue keys (with lower case keys name but unchanged prefix values);
	paramFieldList = "#prefix#" & ListChangeDelims(lcase(StructKeyList(defaultValues,',')),",#prefix#");
</cfscript>

<cfsavecontent variable="cftFooterJS">
<cfoutput>
<script type="text/javascript">
<!--
	<!--- // for validation - 'paddding' is not needed in this list --->
	var cs_propNames = ['tag','align','size','color','bgcolor','bordercolor','style','effect'];
	
	fieldProperties['#typeid#'].paramFields = "#paramFieldList#";
	fieldProperties['#typeid#'].jsValidator = '#prefix#doValidate';

	// setup onload event
	window.onload = function(){ doOnLoad() };	
	
	function doOnLoad()
	{
		<cfloop index="prop" list="#gProps#">
			toggleRow('#prefix#','#prop#');
		</cfloop>
	}

	function toggleRow(prefix, prop) 
	{

		var checked = document.getElementById(prefix + 'display' + prop).checked;

		// Label
		var label = document.getElementById(prefix + prop + 'label');
		if( checked )
		{
			label.style.backgroundColor = '##fff';
			label.disabled = false;
		}
		else
		{
			label.style.backgroundColor = '##f0f0f0';
			label.disabled = true;
		}
		
		// Value
		var valuesList = document.getElementById(prefix + prop + 'value');
		if( checked )
		{
			valuesList.style.backgroundColor = '##fff';
			valuesList.disabled = false;
		}
		else
		{
			valuesList.style.backgroundColor = '##f0f0f0';
			valuesList.disabled = true;
		}
	}

	function #prefix#doValidate()
	{
		// check that all fields except Padding have values if shown
		for (var i = 0; i < cs_propNames.length; i++)
		{
			var cb = document.getElementById('#prefix#display' + cs_propNames[i].toLowerCase());
			if (cb.checked)
			{
				var field = document.getElementById('#prefix#' + cs_propNames[i].toLowerCase() + 'value');
				if (field.value === '')
				{
					showMsg(cs_propNames[i] + ' is required when checked.');
					return false;
				}
			}
		}
		return true;
	}
//-->
</script>
</cfoutput>
</cfsavecontent>

<cfscript>
	application.ADF.scripts.addFooterJS(cftFooterJS,"TERTIARY");
</cfscript>

<cfoutput>

	<cfif Not StructKeyExists(request,'cft_field_settings')>
		<cfoutput>
			<style>
				td.cs_dlgLabelSmall { max-width: 660px; }
				.flexcontainer {
					display: flex;
					flex-wrap: wrap;
					flex-direction: row;
				}
				.flexitem {
					margin-bottom: 5px;
				}
			</style>
		</cfoutput>
		<cfset request.cft_field_settings = 1>
	</cfif>

	<table>
		<tr>
			<td class="cs_dlgLabelBold" valign="top" nowrap="nowrap">Display:</td>
			<td class="cs_dlgLabelSmall" colspan="3">

				<div class="flexcontainer">

				<!--- #renderAttribCheckbox(rowName,rowLabel,renderRow)#  --->
				<!--- 'tag','align','size','color','bgcolor','bordercolor','style','effect','padding' --->
				
				#renderAttribCheckbox("show","Show",defaultValues.displayShow)#
				
				#renderAttribCheckbox("tag","Tag",defaultValues.displayTag)# 

				#renderAttribCheckbox("align","Alignment",defaultValues.displayAlign)#

				#renderAttribCheckbox("size","Size",defaultValues.displaySize)#
				
				#renderAttribCheckbox("fontsettings","Font Settings",defaultValues.displayFontSettings)#

				#renderAttribCheckbox("color","Color",defaultValues.displayColor)#

				#renderAttribCheckbox("bgcolor","BG Color",defaultValues.displayBgColor)#

				#renderAttribCheckbox("bordercolor","Border Color",defaultValues.displayBorderColor)#

				#renderAttribCheckbox("style","Style",defaultValues.displayStyle)#

				#renderAttribCheckbox("effect","Effects",defaultValues.displayEffect)#

				#renderAttribCheckbox("padding","Padding",defaultValues.displayPadding)#
				</div>
			</td>
		</tr>
		
		<tr>
			<td></td>
			<td class="cs_dlgLabelSmall" colspan="3">
				<div>
					For each field below, specify the possible values to appear in its selection list.  
					You can specify either:</div>
					<ul style="margin-top:0;">
						<li>a simple comma delimited list of values</li>
						<li>a comma delimited list of display names and values, in the form displayname=value</li>
						<li>a pointer to a Custom Element from which to get its values, in the form:
							<div style="margin-left: 20px;">element={element name},displayfield={display field name},valuefield={value field name}</div>
						</li>
					</ul>
			</td>
		</tr>
		
		<tr id="#prefix#HeaderRow">
			<td class="cs_dlgLabelSmall" valign="top" nowrap="nowrap"><!--Prop Type--></td>
			<td class="cs_dlgLabelSmallBold" valign="top" nowrap="nowrap">Label</td>
			<td class="cs_dlgLabelSmallBold" valign="top" nowrap="nowrap">Values</td>
			<td class="cs_dlgLabelSmallBold" valign="top" nowrap="nowrap">Default Value</td>
		</tr>
		
		<!--- renderAttribRow(rowName,rowLabel,labelValue,attribValue,defaultValue,renderRow) --->
		<!--- 'tag','align','size','color','bgcolor','bordercolor','style','effect','padding' --->
		
		#renderAttribRow("show","Show",defaultValues.showLabel,defaultValues.showValue,defaultValues.showDefault,true,false,true,false)#
		
		#renderAttribRow("tag","Tag",defaultValues.tagLabel,defaultValues.tagValue,defaultValues.tagDefault,true,false,false,false)#

		#renderAttribRow("align","Alignment",defaultValues.alignLabel,defaultValues.alignValue,defaultValues.alignDefault,true,false,false,false)#

		#renderAttribRow("size","Size",defaultValues.sizeLabel,defaultValues.sizeValue,defaultValues.sizeDefault,true,false,false,false)#

		#renderAttribRow("fontSettings","Font Settings",defaultValues.fontSettingsLabel,defaultValues.fontSettingsValue,defaultValues.fontSettingsDefault,true,false,true,false)#

		#renderAttribRow("color","Color",defaultValues.colorLabel,defaultValues.colorValue,defaultValues.colorDefault,true,false,false,false)#

		#renderAttribRow("bgColor","BG Color",defaultValues.bgColorLabel,defaultValues.bgColorValue,defaultValues.bgColorDefault,true,false,false,false)#

		#renderAttribRow("borderColor","Border Color",defaultValues.borderColorLabel,defaultValues.borderColorValue,defaultValues.borderColorDefault,true,false,false,false)#

		#renderAttribRow("style","Style",defaultValues.styleLabel,defaultValues.styleValue,defaultValues.styleDefault,true,false,false,false)#

		#renderAttribRow("effect","Effects",defaultValues.effectLabel,defaultValues.effectValue,defaultValues.effectDefault,true,false,false,false)#

		#renderAttribRow("padding","Padding",defaultValues.paddingLabel,defaultValues.paddingValue,defaultValues.paddingDefault,true,false,false,false)#
		<tr>
			<td class="cs_dlgLabelSmall" style="font-size:7pt; color:##f0f0f0;" colspan="2">
				ADF Custom Field v#fieldVersion#
			</td>
		</tr>
	</table>
</cfoutput>

<!----------------------------------------
	renderAttribCheckbox(rowName,rowLabel,renderRow)
----------------------------------------->
<cffunction name="renderAttribCheckbox" access="private" output="Yes" returntype="void">
	<cfargument name="rowName" type="string" required="Yes">
	<cfargument name="rowLabel" type="string" required="No" default="#arguments.name#">
	<cfargument name="renderRow" type="boolean" required="No" default="false">
	
	<cfscript>
		var aName = lcase(arguments.rowName);
		var aLabel = arguments.rowLabel;
	</cfscript>
	
	<cfoutput>
		<span class="flexitem" id="#arguments.rowName#_#arguments.rowLabel#_wrapper">
		<input type="checkbox" name="#prefix#display#aName#" id="#prefix#display#aName#" value="1" onclick="toggleRow('#prefix#','#aName#');" class="cs_dlgControl"<cfif arguments.renderRow eq 1> checked="checked"</cfif>><label for="#prefix#display#aName#" style="margin-right: 10px;">#aLabel#</label>
		</span>
	</cfoutput>
</cffunction>

<!----------------------------------------
	renderAttribRow(rowName,rowLabel,labelValue,attribValue,defaultValue,renderRow,labelHide,attribHide,defaultHide)
----------------------------------------->
<cffunction name="renderAttribRow" access="private" output="Yes" returntype="void">
	<cfargument name="rowName" type="string" required="Yes">
	<cfargument name="rowLabel" type="string" required="No" default="#arguments.name#">
	<cfargument name="labelValue" type="string" required="No" default="">
	<cfargument name="attribValue" type="string" required="No" default="">
	<cfargument name="defaultValue" type="string" required="No" default="">
	<cfargument name="renderRow" type="boolean" required="No" default="false">
	<cfargument name="labelHide" type="boolean" required="No" default="false">
	<cfargument name="attribHide" type="boolean" required="No" default="false">
	<cfargument name="defaultHide" type="boolean" required="No" default="false">
	
	<cfscript>
		var aName = lcase(arguments.rowName);
		var aLabel = arguments.rowLabel;
	</cfscript>

	<cfoutput>
		<tr id="#prefix##aName#Row"<cfif arguments.renderRow EQ 0> style="display: none"</cfif>>
			
			<td class="cs_dlgLabelBold" valign="top" nowrap="nowrap">#aLabel#:</td>
			
			<td class="cs_dlgLabelSmall">
				<cfif!arguments.labelHide></cfif>
				<input<cfif !arguments.labelHide> type="text"<cfelse> type="hidden"</cfif> name="#prefix##aName#label" id="#prefix##aName#label" class="cs_dlgControl" value="#arguments.labelValue#" size="10">
			</td>
			
			<td class="cs_dlgLabelSmall">
				<input<cfif !arguments.attribHide> type="text"<cfelse> type="hidden"</cfif> name="#prefix##aName#value" id="#prefix##aName#value" class="cs_dlgControl" value="#arguments.attribValue#" size="85">
			</td>

			<td class="cs_dlgLabelSmall">
				<input<cfif !arguments.defaultHide> type="text"<cfelse> type="hidden"</cfif> name="#prefix##aName#default" id="#prefix##aName#default" class="cs_dlgControl" value="#arguments.defaultValue#" size="15">
			</td>
			
		</tr>
	</cfoutput>

</cffunction>