<!---
The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 
Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.
 
The Original Code is comprised of the ADF directory
 
The Initial Developer of the Original Code is
PaperThin, Inc.  Copyright (c) 2009-2018.
All Rights Reserved.
 
By downloading, modifying, distributing, using and/or accessing any files
in this directory, you agree to the terms and conditions of the applicable
end user license agreement.
--->

<!---
/* *************************************************************** */
Author: 	   				
	PaperThin Inc.
Name:
	field_settings_render.cfc
Summary:
	Renders up to 10 property fields, including:
		tag, align, size, color, bgcolor, bordercolor, style, effect, padding, fontsettings
	
	extends="ADF.extensions.customfields.adf-form-field-renderer-base"
Version:
	1.0
History:
	2017-11-29 - Created
	2017-12-06 - GAC - Updated Label spacing
	2017-12-08 - DRM - Added units support for padding
	2017-12-12 - JTP - Added units title text to selection list
--->
<cfcomponent displayName="Field Settings Render" extends="ADF.extensions.customfields.adf-form-field-renderer-base">

<!--- 'tag','align','size','color','bgcolor','bordercolor','style','effect','padding','fontsettings' --->

<cfset gPropsNoShow = 'tag,align,size,fontsettings,color,bgcolor,bordercolor,style,effect,padding'>
<cfset gProps = 'show,#gPropsNoShow#'>

<cffunction name="renderControl" returntype="void" access="public">
	<cfargument name="fieldName" type="string" required="yes">
	<cfargument name="fieldDomID" type="string" required="yes">
	<cfargument name="value" type="string" required="yes">
	
	<cfscript>
		var inputParameters = application.ADF.data.duplicateStruct(arguments.parameters);
		var currentValue = arguments.value;	// the field's current value
		var inputParameters = setDefaultParameters(argumentCollection=arguments);
		var currValueStruct = structNew();
		var i = 0;
		var prop = '';

		for( i=1; i lte ListLen(gProps); i=i+1 )
		{
			prop = ListGetAt(gProps,i);
			if( NOT StructKeyExists(inputParameters, '#prop#Value') )
				inputParameters['#prop#Value'] = '';
			if( NOT StructKeyExists(inputParameters, '#prop#Display') )	
				inputParameters['#prop#Display'] = '';
			if( NOT StructKeyExists(inputParameters, '#prop#Default') )
				inputParameters['#prop#Default'] = '';
		}		
		
		// force values for Show Property
		inputParameters['showValue'] = 'yes,no';

		if( len(currentValue) AND isJSON(currentValue) )
		{
			currValueStruct = deserializeJSON(currentValue);

			// clear padding out if all 0's
			if( currValueStruct.padding eq '0px 0px 0px 0px' )
				currValueStruct.padding = '';	
			
			for( i=1; i lte ListLen(gProps); i=i+1 )
			{
				prop = ListGetAt(gProps,i);
	
				// set the current value into the props Default setting
				if( structKeyExists(currValueStruct, prop) )
					inputParameters['#prop#Default'] = currValueStruct[prop];
			}
		}
	</cfscript>

	
	<cfif Not StructKeyExists(request,'cft_field_settings')>
		<cfoutput>
			<style>
				/** this max-width causes scroll bars on first tabs if this field is on a second tab 
					##cs_commondlg .CS_FormFieldControlContainer { max-width: 660px; } **/
				.fsFlexcontainer {
					display: flex;
					flex-wrap: wrap;
					flex-direction: row;
					margin-left: -5px;
				}
				.fsFlexitem {
					margin-bottom: 5px;
				}
				.fsFlexitem > span {
					padding-left: 5px;
				}
				.fsPaddingGroup {
					padding-left: 5px;
				}
				.fsPaddingItemLabel {
					padding-left: 5px;
					text-transform: capitalize;
				}

				.fsFontBtn { 
					display: inline-block;
					background-color: ##fff; 
					border: 1px solid ##999; 
					box-sizing: border-box; 
			    	padding: 5px 5px;
    				height: 21px;
				}
				.fsFontBtn:hover {
					background-color: ##eee;
				}
				.fsFontBtn.active:hover {
					background-color: ##aaa;
				}				
				.fsFontBtn.active { background-color: ##ddd; }
			</style>
		</cfoutput>
		<cfset request.cft_field_settings = 1>
	</cfif>

	<!---
		Render the field using a flex box container to wrap on each row
	--->	
<cftry>

	<cfoutput><div class="fsFlexcontainer"></cfoutput>
		
			<cfscript>
				// loop over each property
				for( i=1; i lte ListLen(gProps); i=i+1 )
				{
					prop = ListGetAt(gProps, i);				
					RenderProperty(inputParameters, prop, currValueStruct);
				}
			</cfscript>

		<cfoutput>
			<!--- // hidden field to store the value --->
			<input type="hidden" name="#arguments.fieldName#" value='#currentValue#' id="#inputParameters.fieldID#" class="#inputParameters.fieldClass#">
		</div>
		</cfoutput>
	
	<cfscript>
		renderJSFunctions(argumentCollection=arguments,fieldParameters=inputParameters);
	</cfscript>
	
<cfcatch>
	<cfdump var="#cfcatch#" expand="no" label="ERROR">
</cfcatch>
</cftry>
	
</cffunction>


<cffunction name="RenderProperty" access="private" output="Yes" returntype="void">
	<cfargument name="params" required="Yes" type="struct">
	<cfargument name="prop" required="Yes" type="string">
	<cfargument name="currentValuesStruct" required="Yes" type="struct">
	
	<cfscript>
		var inputParameters = arguments.params;
		var possibleValuesList = inputParameters['#arguments.prop#Value'];
		var possibleDisplayList = '';
		var defaultValue = inputParameters['#arguments.prop#Default'];			
		var elData = '';
		var isColorList = 0;
		var renderAsList = 0;
		var i = 0;
		var vList = '';
		var dList = '';
		var item = '';
		var separator = ',';		// assume comma separator, except when from Custom element use Pipe 

		var value = defaultValue;
		
		// determine if Props is for color		
		if( FindNoCase( 'color', arguments.prop ) )
			isColorList = 1;

		/*writedump(var=arguments.prop,label="arguments.prop",expand="no");*/
		/*writeoutput('arguments.prop: #arguments.prop#');*/

		// Check if Property is to be displayed
		if( structKeyExists(inputParameters, 'display#arguments.prop#') AND 
				inputParameters['display#arguments.prop#'] eq 1 )
		{
			if( Len(possibleValuesList) eq 0 )
			{
				// Empty list - assume user can enter data manually so display input control
				renderAsList = 0;
			}
			
			// Check if List comes from Custom Element data
			else if( FindNoCase('element=', possibleValuesList) )
			{
				renderAsList = 1;
				separator = '|';
				
				try 
				{
					// Lookup values
					elData = getElementValues( possibleValuesList );

					if( len(elData.error) )
						possibleValuesList = elData.error;
					else
					{
						possibleValuesList = elData.valuelist;
						possibleDisplayList = elData.displaylist;
					}
				}
				catch(any e)
				{
					WriteOutput('<p>There is an error in your properties configuration for #arguments.prop#. Please review it and try it again. The error message was:<br>#cfcatch.message#</p>');
				}	
			}	
			
			// Assume list is a comma delimited list, NOT empty and not pointer Custom Element
			else if( ListLen(possibleValuesList) gt 0 ) 
			{
				renderAsList = 1;
				 
				// check if the possible values list is in the form: displayvalue=value
				if( FindNoCase( '=', possibleValuesList) )
				{
					for( i=1; i lte ListLen(possibleValuesList); i = i+ 1 )
					{
						item = ListGetAt(possibleValuesList, i);
						dList = ListAppend( dList, ListFirst(item, '=') );
						vList = ListAppend( vList, ListLast(item, '=') );
					}
					
					// reset lists
					possibleValuesList = vList;
					possibleDisplayList = dList;
				}
				else	// normal comma delimited list
				{
					possibleDisplayList = '';
				}
				
			}
			else	// Treat as empty list
			{
				renderAsList = 0;
			}
			
			// Render Selection List
			if( renderAsList )
			{
				if( arguments.prop eq 'padding' )
				{
					RenderPaddingInput( 
						show = inputParameters['display#arguments.prop#'],
						label = inputParameters['#arguments.prop#Label'],
						FieldID = inputParameters.FieldID,
						value = value, 
						possibleValuesList = possibleValuesList,
						possibleDisplayList = possibleDisplayList);
				}
				else
				{
					RenderSelectionList(
						show = inputParameters['display#arguments.prop#'],
						label = inputParameters['#arguments.prop#Label'] & ":",
						FieldID = inputParameters.FieldID,
						name = arguments.prop,
						value = value,
						possibleDisplays = possibleDisplayList,
						possibleValues = possibleValuesList,
						isColorList = isColorList,
						separator = separator );
				}
			}
			else
			{
				// value in text field
				if( arguments.prop eq 'padding' )
				{
					RenderPaddingInput( 
						show = inputParameters['display#arguments.prop#'],
						label = inputParameters['#arguments.prop#Label'],
						FieldID = inputParameters.FieldID,
						value = value, 
						possibleValuesList = possibleValuesList,
						possibleDisplayList = possibleDisplayList,
						className = 'paddingItem' );
				}

				else if( arguments.prop eq "fontsettings" )
				{
					RenderFontSettingsInput(
						show = inputParameters['display#arguments.prop#'],
						label = inputParameters['#arguments.prop#Label'],
						FieldID = inputParameters.FieldID,
						value = value);
				}

				else
				{
					WriteOutput( inputParameters['#arguments.prop#Label'] & ":" );
					WriteOutput('<input type="text" name="#arguments.prop#" value="#value#" style="width:50px;">');
				}
			}
		}
		else
		{
			// Render default value in hidden field
			WriteOutput('<input type="hidden" name="#arguments.prop#" value="#defaultValue#">');
		}	
	</cfscript>	

</cffunction>


<cffunction name="RenderFontSettingsInput" access="private" output="Yes" returntype="void">
	<cfargument name="label" type="string" required="Yes">
	<cfargument name="fieldID" type="string" required="Yes">
	<cfargument name="value" type="string" required="Yes">

	<cfscript>
		var fontDefaultProps = 'bold:0,italic:0,underline:0,case:none';
		var currSetting = "";
		var currVal = "";
		var boldActive = "";
		var italicActive = "";
		var underlineActive = "";
		var caseNone = "";
		var caseCap = "";
		var caseUpper = "";
		var caseLower = "";
		var i = 0;
		var itm = '';
		var fontConfigArray = arraynew(1);
		
		if( len(arguments.value) == 0 )
			arguments.value = fontDefaultProps;

		if( len(arguments.value) )
		{
			// build array from values
			for( i=1; i lte listlen(arguments.value); i=i+1 )
			{
				itm = ListGetAt(arguments.value, i);
				currSetting = listGetAt(itm,1,":");
				currValue = listGetAt(itm,2,":");
				fontConfigArray[i] = structNew();
				fontConfigArray[i].setting = currSetting;
				fontConfigArray[i].value = currValue;
			}
		}

		if( ArrayLen(fontConfigArray) gte 1 AND fontConfigArray[1].value EQ 1 )
			boldActive = "active";
		if( ArrayLen(fontConfigArray) gte 2 AND fontConfigArray[2].value EQ 1 )
			italicActive = "active";
		if( ArrayLen(fontConfigArray) gte 3 AND fontConfigArray[3].value EQ 1 )
			underlineActive = "active";

		if( ArrayLen(fontConfigArray) gte 4 )
		{
			if( fontConfigArray[4].value eq "none" )
				caseNone = "selected";
			if( fontConfigArray[4].value eq "capitialize" )
				caseCap = "selected";
			if( fontConfigArray[4].value eq "uppercase" )
				caseUpper = "selected";
			if( fontConfigArray[4].value eq "lowercase" )
				caseLower = "selected";
		}
				
	</cfscript>
	
	<cfoutput>
		<span class="fsFlexitem" id="#arguments.fieldID#_fontsettings_wrapper">
			<cfif LEN(TRIM(arguments.label))><span>#arguments.label#:</span></cfif>
			</cfoutput>
			
			<cfoutput><div class="fsFontBtn #boldActive#" onclick="clickFontButton($(this),'bold', '#arguments.fieldID#');">
				<i class="fa fa-bold"></i>
			</div></cfoutput>
			<cfoutput><div class="fsFontBtn #italicActive#" onclick="clickFontButton($(this),'italic', '#arguments.fieldID#');">
				<i class="fa fa-italic"></i>
			</div></cfoutput>
			<cfoutput><div class="fsFontBtn #underlineActive#" onclick="clickFontButton($(this),'underline', '#arguments.fieldID#');">
				<i class="fa fa-underline"></i>
			</div>
			<select id="#arguments.fieldID#_case" name="#arguments.fieldID#_case" onchange="caseChange($(this),'#arguments.fieldID#')">
				<option value="none" #caseNone#>Initial Case</option>
				<option value="capitialize" #caseCap#>Title Case</option>
				<option value="uppercase" #caseUpper#>Upper Case</option>
				<option value="lowercase" #caseLower#>Lower Case</option>
			</select>
	
			<!--- hidden field to store value --->
			<input type="hidden" id="#arguments.fieldID#_fontsettings" name="#arguments.fieldID#_fontsettings" value="#arguments.value#">
		</span>
	</cfoutput>

</cffunction>


<cffunction name="RenderPaddingInput" access="private" output="Yes" returntype="void">
	<cfargument name="label" type="string" required="Yes">
	<cfargument name="fieldID" type="string" required="Yes">
	<cfargument name="value" type="string" required="Yes">
	<cfargument name="possibleValuesList" type="string" required="No" default="">
	<cfargument name="possibleDisplayList" type="string" required="No" default="">
	
	<cfscript>
		var pos = ['top','right', 'bottom', 'left'];
		var i = 0;
		var id = '';
		var lbl = '';
		var itemValue = '';
		var unitsOptions = 'px,%,em,rem,vw,vh,vmin,vmax,none';	
		var units = 'none';
		var v = StructNew();
		
		var unitsHints = ArrayNew(1);
		var unitsHintsList = "";
		
		unitsHints[1] = 'Spacing in pixels';
		unitsHints[2] = 'Spacing relative to the percentage of the parent';		
		unitsHints[3] = 'Spacing relative to the font-size of the element (2em = 2 times the size of the current font)';
		unitsHints[4] = 'Spacing relative to font-size of the root element';
		unitsHints[5] = 'Spacing relative to 1% of the width of the viewport';
		unitsHints[6] = 'Spacing relative to 1% of the height of the viewport';		
		unitsHints[7] = "Spacing relative to 1% of viewport's smaller dimension";		
		unitsHints[8] = "Spacing relative to 1% of viewport's larger dimension";		
		unitsHints[9] = "Don't set spacing";	
		
		unitsHintsList = ArrayToList(unitsHints);				
		
		v.top = '0';
		v.right = '0';
		v.bottom = '0';
		v.left = '0';

		
		// remove px's
		arguments.value = Trim(ReplaceNoCase( arguments.value, 'px', '', 'ALL' ));
		
		if( ListLen(arguments.value, ' ') eq 4 )
		{
			v.top = val(ListGetAt(arguments.value,1, ' '));
			v.right = val(ListGetAt(arguments.value,2, ' '));
			v.bottom = val(ListGetAt(arguments.value,3, ' '));
			v.left = val(ListGetAt(arguments.value,4, ' '));
			units = reReplace(listFirst(arguments.value, " "), "[0-9]", "", "ALL");
		}
		
		// Output padding wrapper 
		WriteOutput( '<div class="fsPaddingGroup" id="#arguments.fieldID#_padding_wrapper">');
		
		// Output padding label
		WriteOutput( '<span>');
		WriteOutput( arguments.label & ':');
		// Added &nbsp; to fix css capitalize from seeing label/sublabel as one word
		WriteOutput( '&nbsp;'); 
		WriteOutput( '</span>');

		RenderSelectionList(
			show = 1,
			label = 'Units',
			FieldID = arguments.FieldID,
			name = 'padding_Units',
			value = units,
			possibleDisplays = unitsOptions,
			possibleValues = unitsOptions,
			isColorList = 0,
			hints = unitsHintsList );

		for( i=1; i lte ArrayLen(pos); i=i+1 )
		{
			itemValue = v[pos[i]];
			
			// sublabel
			WriteOutput( '<span class="fsPaddingItemLabel">' );
			WriteOutput( pos[i] & ':');
			WriteOutput( '</span>' );

			if( arguments.possibleValuesList eq '' )
			{
				onChangeFunc = "onChangeFieldSettingProp('#arguments.fieldID#')";
				id = '#arguments.fieldID#_padding_#pos[i]#';
				WriteOutput('<input type="number" id="#id#" name="#id#" value="#itemValue#" class="paddingItem" style="width:40px;text-align:right;margin-right:3px;padding:2px 0 2px 2px;" size="3" min="0" max="999" onchange="#onChangeFunc#">');
			}
			else
			{
				RenderSelectionList(
					show = 1,
					label = '',
					FieldID = arguments.FieldID,
					name = 'padding_#pos[i]#',
					value = itemValue,
					possibleDisplays = arguments.possibleDisplayList,
					possibleValues = arguments.possibleValuesList,
					isColorList = 0,
					className = "paddingItem",
					style="text-align:right;margin-right:0;" );
			}	
			
			//WriteOutput( '#units# ');
		}
		
		WriteOutput('</div>');
	</cfscript>
</cffunction>

<cffunction name="renderJSFunctions" returntype="void" access="private">
	<cfargument name="fieldName" type="string" required="yes">
	<cfargument name="fieldDomID" type="string" required="yes">
	<cfargument name="value" type="string" required="yes">
	<cfargument name="fieldParameters" type="struct" required="yes">
	
	<cfscript>
		var inputParameters = application.ADF.data.duplicateStruct(arguments.fieldParameters);
		var cftFooterJS = '';
	</cfscript>

	<cfif NOT StructKeyExists(request,'renderedFieldSettingJS')>
		<cfset request.renderedFieldSettingJS = 1>

		<cfsavecontent variable="cftFooterJS">
			<cfoutput>
			<script type="text/javascript">
			function onChangeFieldSettingProp(fieldID)
			{
				var paddingUnits, showPaddingFields, selector;
				if( jQuery('##' + fieldID + '_show').length ) 
				{
					var currShow = jQuery('##' + fieldID + '_show').val();
			
					if( currShow === "yes" ) 
					{
						<cfloop index="prop" list="#gPropsNoShow#">
							if( jQuery('##' + fieldID + '_#prop#_wrapper').length )
								jQuery('##' + fieldID + '_#prop#_wrapper').css("display","inline-block");
						</cfloop>
					} 
					else 
					{
						<cfloop index="prop" list="#gPropsNoShow#">
							if( jQuery('##' + fieldID + '_#prop#_wrapper').length ) 
								jQuery('##' + fieldID + '_#prop#_wrapper').css("display","none");
						</cfloop>
					}
				}
				else 
				{
					var currshow = '';
				}
			
				<cfloop index="prop" list="#gPropsNoShow#">
					<cfif prop eq 'padding'>
						paddingUnits = jQuery('##' + fieldID + '_padding_Units').val();
						showPaddingFields = (paddingUnits !== 'none') ;
						selector = '##' + fieldID + '_padding_wrapper .paddingItem,##' + fieldID + '_padding_wrapper .fsPaddingItemLabel';
						jQuery(selector).toggle(showPaddingFields);
						<cfloop index="pos" list="top,right,bottom,left">
							if( jQuery('##' + fieldID + '_#prop#_#pos#').length && paddingUnits !== 'none' )
								var curr#prop#_#pos# = jQuery('##' + fieldID + '_#prop#_#pos#').val();
							else
								var curr#prop#_#pos# = '';
						</cfloop>	
					<cfelse>
						if( jQuery('##' + fieldID + '_#prop#').length ) 
							var curr#prop# = jQuery('##' + fieldID + '_#prop#').val();
						else 
							var curr#prop# = '';
					</cfif>	
				</cfloop>
				
				var pad = normalizePaddingValue(paddingUnits, currpadding_top, currpadding_right, currpadding_bottom, currpadding_left);

				var currObj = {"show":currShow,"tag":currtag,"align":curralign,"size":currsize,"color":currcolor,"bgcolor":currbgcolor,"bordercolor":currbordercolor,"style":currstyle,"effect":curreffect,"padding":pad,"fontsettings":currfontsettings};
				var currJSON = JSON.stringify(currObj);
				
				//	console.log(currJSON);

				// Set value into field				
				jQuery('##' + fieldID ).val(currJSON);
			}

			// init display
			onChangeFieldSettingProp('#arguments.fieldDomID#');
			
			function normalizePaddingValue(units,t,r,b,l)
			{
				var number = 0;
				var retStr = '';

				if (units === '')
					return '';
				
				// We assume Numbers ONLY. If not make them numbers
				
				number = parseInt(t);
				t = isNaN(number) ? 0 : number;
				
				number = parseInt(r);
				r = isNaN(number) ? 0 : number;
				
				number = parseInt(b);
				b = isNaN(number) ? 0 : number;

				number = parseInt(l);
				l = isNaN(number) ? 0 : number;
				
				if( t===0 && r===0 && b===0 && l===0 )
					retStr = '';
				else
					retStr = t + units + ' ' + r + units + ' ' + b + units + ' ' + l + units;
					
				return retStr;
			}
			
			function clickFontButton(el, btnName, fldID)
			{
				var hiddenFld = jQuery( '##' + fldID + '_fontsettings' );
				var fontProps = hiddenFld.val();	
				var array = fontProps.split(',');
				
				if( btnName === 'bold' )
					i = 0;
				else if( btnName === 'italic' )
					i = 1;
				else 
					i = 2;
		
				btn = jQuery(el);
				
				if( btn.hasClass('active') )
				{
					btn.removeClass('active');
					array[i] = btnName + ':'  + 0;
				}
				else
				{
					btn.addClass('active');
					array[i] = btnName + ':'  + 1;
				}
				hiddenFld.val( array.toString() );
				onChangeFieldSettingProp(fldID);
			}
			
			function caseChange(el, fldID)
			{
				var caseFld = jQuery( '##' + fldID + '_case' );
				var caseValue = caseFld.val();
				
				var hiddenFld = jQuery( '##' + fldID + '_fontsettings' );
				var fontProps = hiddenFld.val();	
				var array = fontProps.split(',');
				
				array[3] = 'case:' + caseValue;
				
				hiddenFld.val( array.toString() );
				onChangeFieldSettingProp(fldID);
			}
			</script>
			</cfoutput>
		</cfsavecontent>		
		
	</cfif>	

	<cfscript>
		application.ADF.scripts.addFooterJS(cftFooterJS,"TERTIARY");
	</cfscript>

	<cfsavecontent variable="cftFooterJS">
		<cfoutput>
			<script type="text/javascript">
				onChangeFieldSettingProp('#inputParameters.fieldID#');
			</script>
		</cfoutput>
	</cfsavecontent>

	<cfscript>
		application.ADF.scripts.addFooterJS(cftFooterJS,"TERTIARY");
	</cfscript>
</cffunction>


<cffunction name="setDefaultParameters" returntype="struct" access="private">
	<cfargument name="fieldName" type="string" required="yes">
	<cfargument name="fieldDomID" type="string" required="yes">
	<cfargument name="value" type="string" required="yes">
	
	<cfscript>
		var inputParameters = application.ADF.data.duplicateStruct(arguments.parameters);
		
		// Validate if the property field has been defined
		if ( NOT StructKeyExists(inputParameters, "fldID") OR LEN(inputParameters.fldID) LTE 0 )
			inputParameters.fldID = arguments.fieldName;
		if ( NOT StructKeyExists(inputParameters, "fieldID") )
			inputParameters.fieldID = arguments.fieldName;
		if ( (NOT StructKeyExists(inputParameters, "fieldClass")) OR ( LEN(TRIM(inputParameters.fieldClass)) LTE 0) )
			inputParameters.fieldClass = "";

		if ( NOT StructKeyExists(inputParameters, "displayShow") )
			inputParameters.displayShow = 0;
		if ( NOT StructKeyExists(inputParameters, "displayTag") )
			inputParameters.displayTag = 0;
		if ( NOT StructKeyExists(inputParameters, "displayAlign") )
			inputParameters.displayAlign = 0;
		if ( NOT StructKeyExists(inputParameters, "displaySize") )
			inputParameters.displaySize = 0;
		if ( NOT StructKeyExists(inputParameters, "displayColor") )
			inputParameters.displayColor = 0;
		if ( NOT StructKeyExists(inputParameters, "displaybgColor") )
			inputParameters.displayBgColor = 0;
		if ( NOT StructKeyExists(inputParameters, "displayBorderColor") )
			inputParameters.displayBorderColor = 0;
		if ( NOT StructKeyExists(inputParameters, "displayStyle") )
			inputParameters.displayStyle = 0;	
		if ( NOT StructKeyExists(inputParameters, "displayEffect") )
			inputParameters.displayEffect = 0;		
		if ( NOT StructKeyExists(inputParameters, "displayPadding") )
			inputParameters.displayPadding = 0;
		if ( NOT StructKeyExists(inputParameters, "displayFontSettings") )
			inputParameters.displayFontSettings = 0;

		if ( NOT StructKeyExists(inputParameters, "showLabel") OR LEN(TRIM(inputParameters.showLabel)) eq 0 )
			inputParameters.showLabel = "Show";
		if ( NOT StructKeyExists(inputParameters, "tagLabel") OR LEN(TRIM(inputParameters.tagLabel)) eq 0 )
			inputParameters.tagLabel = "Tag";
		if ( NOT StructKeyExists(inputParameters, "alignLabel") OR LEN(TRIM(inputParameters.alignLabel)) eq 0 )
			inputParameters.alignLabel = "Align";
		if ( NOT StructKeyExists(inputParameters, "sizeLabel") OR LEN(TRIM(inputParameters.sizeLabel)) eq 0 )
			inputParameters.sizeLabel = "Size";
		if ( NOT StructKeyExists(inputParameters, "colorLabel") OR LEN(TRIM(inputParameters.colorLabel)) eq 0 )
			inputParameters.colorLabel = "Color";
		if ( NOT StructKeyExists(inputParameters, "bgColorLabel") OR LEN(TRIM(inputParameters.bgColorLabel)) eq 0 )
			inputParameters.bgColorLabel = "Background Color";
		if ( NOT StructKeyExists(inputParameters, "borderColorLabel") OR LEN(TRIM(inputParameters.borderColorLabel)) eq 0 )
			inputParameters.borderColorLabel = "Border Color";
		if ( NOT StructKeyExists(inputParameters, "styleLabel") OR LEN(TRIM(inputParameters.styleLabel)) eq 0 )
			inputParameters.styleLabel = "Style";
		if ( NOT StructKeyExists(inputParameters, "effectLabel") OR LEN(TRIM(inputParameters.effectLabel)) eq 0 )
			inputParameters.effectLabel = "Effect";
		if ( NOT StructKeyExists(inputParameters, "paddingLabel") OR LEN(TRIM(inputParameters.paddingLabel)) eq 0 )
			inputParameters.paddingLabel = "Padding";
		if ( NOT StructKeyExists(inputParameters, "fontSettingsLabel") OR LEN(TRIM(inputParameters.fontSettingsLabel)) eq 0 )
			inputParameters.fontSettingsLabel = "Font Settings";

		if ( NOT StructKeyExists(inputParameters, "showDefault") )
			inputParameters.showDefault = 1;
		if ( NOT StructKeyExists(inputParameters, "tagDefault") )
			inputParameters.tagDefault = "";
		if ( NOT StructKeyExists(inputParameters, "alignDefault") )
			inputParameters.alignDefault = "";
		if ( NOT StructKeyExists(inputParameters, "sizeDefault") )
			inputParameters.sizeDefault = "";
		if ( NOT StructKeyExists(inputParameters, "colorDefault") )
			inputParameters.colorDefault = "";
		if ( NOT StructKeyExists(inputParameters, "bgColorDefault") )
			inputParameters.bgColorDefault = "";
		if ( NOT StructKeyExists(inputParameters, "borderColorDefault") )
			inputParameters.borderColorDefault = "";
		if ( NOT StructKeyExists(inputParameters, "styleDefault") )
			inputParameters.styleDefault = "";	
		if ( NOT StructKeyExists(inputParameters, "effectDefault") )
			inputParameters.effectDefault = "";		
		if ( NOT StructKeyExists(inputParameters, "paddingDefault") )
			inputParameters.paddingDefault = "";
		if ( NOT StructKeyExists(inputParameters, "fontSettingsDefault") )
			inputParameters.fontSettingsDefault = "";

		// Remove any labels from the possibleValues String
		if ( NOT StructKeyExists(inputParameters, "possibleValues") )
			inputParameters.possibleValues = "";
		
		return inputParameters;
	</cfscript>
</cffunction>


<cffunction name="getElementValues" returntype="struct" access="private">
	<cfargument name="elementConfig" type="string" required="yes">

	<!--- element=Photo Size,displayfield=title,valuefield=sizeid  --->
	<cfscript>
		var currValueList = "";
		var currDisplayList = "";
		var currListStruct = structNew();
		
		var assocArray = StructNew();
		var v = '';
		var d = '';
		var i = 0;
		
		var checkElement = '';
		var currElement = '';
		var currElDisplay = '';
		var currElValue = '';
		var currElementData = '';
		
		var v = '';
		var d = '';
		var key = '';
	</cfscript>

	<cfloop list="#arguments.elementConfig#" item="listitem">
		<cfscript>
			if( listitem contains "element=" )
				currElement = listlast(listitem,"=");
			else if( listitem contains "displayfield=" )
				currElDisplay = listlast(listitem,"=");
			else if( listitem contains "valuefield=" )
				currElValue = listlast(listitem,"=");
		</cfscript>
	</cfloop>

	<cfset checkElement = application.adf.cedata.elementExists(currElement)>

	<cfif checkElement>
		<cfscript>
			currElementData = application.adf.cedata.getCEData(currElement);
			
			for( i=1; i lte ArrayLen(currElementData); i=i+1 )
			{
				v = currElementData[i].values[currElValue];
				d = currElementData[i].values[currElDisplay];
				if( d eq '' )
					d = v;
				assocArray[v] = d;
			}
			sortedKeys = StructSort( assocArray, 'textnocase', 'asc' );

			for( i=1; i lte ArrayLen(sortedKeys); i=i+1 )
			{
				key = sortedKeys[i];
				currValueList = ListAppend(currValueList, key, '|');
				currDisplayList = ListAppend(currDisplayList, assocArray[key], '|');				
			}

			currListStruct.valuelist = currValueList;
			currListStruct.displaylist = currDisplayList;
			currListStruct.error = "";
		</cfscript>
		
	<cfelse>
		<cfset currListStruct.error = "Element '#currElement#' does not exist. Please check your properties configuration and try again.">
	</cfif>

	<cfreturn currListStruct>

</cffunction>


<cfscript>
	/*
		IMPORTANT: Since loadResourceDependencies() is using ADF.scripts loadResources methods, getResourceDependencies() and
		loadResourceDependencies() must stay in sync by accounting for all of required resources for this Custom Field Type.
	*/
	public void function loadResourceDependencies()
	{
		// Load registered Resources via the ADF scripts_2_0
		application.ADF.scripts.loadJQuery(noConflict=true);
		application.ADF.scripts.loadFontAwesome();
	}

	public string function getResourceDependencies()
	{
		return "jQuery,FontAwesome";
	}
	
	public numeric function getMinWidth()
	{
		return 900;
	}
</cfscript>

<!---
/* *************************************************************** */
Author:
	PaperThin, Inc.
Name:
	$RenderSelectionList
Summary:
	Renders a selection list of possible field values.
Returns:
	Void
Arguments:
	Boolean - Show
	String - Label
	String - FieldID
	String - Name
	String - Value
	String - possibleValues
	String - possibleDisplays
	Boolean - isColorList
	String - style
History:
	2017-11-10 - Created
--->

<cffunction name="RenderSelectionList" access="private" output="Yes" returntype="void" hint="renders a selection list of possible padding values.">
	<cfargument name="Show" type="Boolean" required="Yes">
	<cfargument name="Label" type="String" required="Yes">
	<cfargument name="FieldID" type="string" required="Yes">
	<cfargument name="name" type="string" required="Yes">
	<cfargument name="value" type="string" required="Yes">
	<cfargument name="possibleValues" type="string" required="Yes">
	<cfargument name="possibleDisplays" type="string" required="No" default="">
	<cfargument name="isColorList" type="boolean" required="no" default="0">
	<cfargument name="style" type="string" required="no" default="">
	<cfargument name="className" type="string" required="no" default="">
	<cfargument name="separator" type="string" required="no" default=",">
	<cfargument name="hints" type="string" required="no" default="">
	
	<cfscript>
		var sep = arguments.separator;
		var index="";
		var currListValue = "";
		var currListDisplay = "";
		var extra = '';
		var itm = 0;
		var c = '';
		var StyleAttr = '';
		var classAttr = '';
		var bAddHint = 0;
		var cnt = 1;

		if( arguments.possibleDisplays eq '' )
			arguments.possibleDisplays = arguments.possibleValues;
			
		if ( LEN(TRIM(arguments.style)) ) 
			StyleAttr = 'style="#arguments.style#"';

		if (len(arguments.className))
			classAttr = ' class=#arguments.className#';
	</cfscript>

	<cfif arguments.show>
		<cfoutput>
			<span class="fsFlexitem" id="#arguments.FieldID#_#arguments.Name#_wrapper">
				<cfif LEN(TRIM(arguments.label))><span>#arguments.label#</span></cfif>
				<select id="#arguments.FieldID#_#arguments.Name#" name="#arguments.FieldID#_#arguments.Name#"#classAttr# #StyleAttr# onchange="onChangeFieldSettingProp('#FieldID#');">
					</cfoutput>
					
					<cfif arguments.isColorList>
						<cfoutput><option value=""<cfif TRIM(arguments.value) eq ""> selected="selected"</cfif>>not set</option></cfoutput>
					</cfif>
					
					<cfscript>
						bAddHint = 0;
						if( arguments.hints neq '' AND ListLen(arguments.hints) EQ ListLen(arguments.possibleValues ) )
							bAddHint = 1;
					</cfscript>					
					
					<cfloop from="1" to="#listlen(arguments.possibleValues, sep)#" index="itm">
					
						<cfscript>
							currListValue = listGetAt(arguments.possibleValues, itm, sep);
							currListDisplay = listGetAt(arguments.possibleDisplays, itm, sep);

							extra = '';
							if( arguments.isColorList )
							{
								if( FindNoCase('black', currListDisplay) OR FindNoCase('dark', currListDisplay) 
								     OR FindNoCase('navy', currListDisplay) OR FindNoCase('maroon', currListDisplay) 
									  OR FindNoCase('brown', currListDisplay) OR FindNoCase('purple', currListDisplay) )
									c = 'white';
								else
									c = 'black';
								extra = 'class="optColor" style="background-color:#currListValue#;color:#c#;"';
							}
							
							if( bAddHint )
							{
								extra = trim( extra & ' title="#ListGetAt(arguments.hints, cnt)#"' );
								cnt = cnt + 1;	
							}
							
						</cfscript>

						<cfoutput><option #extra# value="#currListValue#" <cfif TRIM(arguments.value) eq TRIM(currListValue)> selected="selected"</cfif>>#TRIM(currListDisplay)#</option></cfoutput>

					</cfloop>

				<cfoutput>
				</select>
			</span>
		</cfoutput>
	<cfelse>
		<cfoutput><input type="hidden" id="#arguments.FieldID#_#arguments.Name#" name="#arguments.FieldID#_#arguments.Name#" value="#TRIM(arguments.value)#"></cfoutput>
	</cfif>
</cffunction>

<!---
/* *************************************************************** */
Author:
	PaperThin, Inc.
Name:
	$RenderInputField
Summary:
	Renders an generic input field
Returns:
	Void
Arguments:
	Boolean - Show
	String - Label
	String - FieldID
	String - Name
	String - Value
History:
	2014-09-15 - Created
--->
<cffunction name="RenderTextInput" access="private" output="Yes" returntype="void" hint="renders a selection list of possible padding values.">
	<cfargument name="Show" type="Boolean" required="Yes">
	<cfargument name="Label" type="String" required="Yes">
	<cfargument name="FieldID" type="string" required="Yes">
	<cfargument name="name" type="string" required="Yes">
	<cfargument name="value" type="string" required="Yes">

	<cfif arguments.show>
		<cfoutput>
			<cfif LEN(TRIM(arguments.label))><span>#arguments.label#</span></cfif>
			<input type="text" id="#arguments.FieldID#_#arguments.Name#" name="#arguments.FieldID#_#arguments.Name#" value="#TRIM(arguments.value)#" size="4" onKeyUp="onChange_#FieldID#();" style="text-align:right;">
		</cfoutput>
	<cfelse>
		<cfoutput><input type="hidden" id="#arguments.FieldID#_#arguments.Name#" name="#arguments.FieldID#_#arguments.Name#" value="#TRIM(arguments.value)#"></cfoutput>
	</cfif>
</cffunction>

<!---
/* *************************************************************** */
Author: 	
	PaperThin, Inc. 
Name:
	$RenderNumberInput
Summary:
	Renders an generic input field 
Returns:
	Void
Arguments:
	Boolean - Show
	String - Label
	String - FieldID
	String - Name
	String - Value
History:
	2017-11-13 - Created
--->
<cffunction name="RenderNumberInput" access="private" output="Yes" returntype="void" hint="renders a selection list of possible padding values.">
	<cfargument name="Show" type="Boolean" required="Yes">
	<cfargument name="Label" type="String" required="Yes">
	<cfargument name="FieldID" type="string" required="Yes">
	<cfargument name="name" type="string" required="Yes">
	<cfargument name="value" type="string" required="Yes">
	
	<cfif arguments.show>
		<cfoutput>
			<cfif LEN(TRIM(arguments.label))><span>#arguments.label#</span></cfif>
			<input type="number" id="#arguments.FieldID#_#arguments.Name#" name="#arguments.FieldID#_#arguments.Name#" value="#TRIM(arguments.value)#" size="3" min="0" max="999" onchange="onChange_#FieldID#();" style="width:40px;text-align:right;margin-right:3px;">px&nbsp;&nbsp;
		</cfoutput>	
	<cfelse>
		<cfoutput><input type="hidden" id="#arguments.FieldID#_#arguments.Name#" name="#arguments.FieldID#_#arguments.Name#" value="#TRIM(arguments.value)#"></cfoutput>
	</cfif>
</cffunction>

</cfcomponent>
